'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _toArray(arr) { return Array.isArray(arr) ? arr : Array.from(arr); }

var _require = require('../string'),
    leftpad = _require.leftpad;

/**
 * Time related utilities
 * @module Time
 */

/**
 * Months labels
 * @member {Object} MONTHS
 * @example
 * MONTHS['mx'][0] //=> "Enero"
 */


var MONTHS = {
  'mx': ['Enero', 'Febrero', 'Marzo', 'Abril', 'Mayo', 'Junio', 'Julio', 'Agosto', 'Septiembre', 'Octubre', 'Noviembre', 'Diciembre']
};

/**
 * Months labes in 3 chars style
 * @member {Object} MONTHS_SHORT
 * @example
 * MONTHS_SHORT['mx'][0] //=> "Ene"
 */
var MONTHS_SHORT = {
  'mx': ['Ene', 'Feb', 'Mar', 'Abr', 'May', 'Jun', 'Jul', 'Ago', 'Sep', 'Oct', 'Nov', 'Dic']
};

/**
 * Days of week labels
 * @member {Object} WEEKDAYS_LONG
 * @example
 * WEEKDAYS_LONG['mx'][0] //=> "Domingo"
 */
var WEEKDAYS_LONG = {
  'mx': ['Domingo', 'Lunes', 'Martes', 'Miércoles', 'Jueves', 'Viernes', 'Sabado']
};

/**
 * Days of week labels in 2 chars style
 * @member {Object} WEEKDAYS_SHORT
 * @example
 * WEEKDAYS_SHORT['mx'][0] //=> "Do"
 */
var WEEKDAYS_SHORT = {
  'mx': ['Do', 'Lu', 'Ma', 'Mi', 'Ju', 'Vi', 'Sa']
};

/* ---end of members--- */

/**
 * Concatenates Year, Month, Day, Hour and Mins into a string to form a current time serial
 *
 * @example
 * dateBasedSerial(new Date(2018, 1, 2, 3, 4)); //=> '201802020304'
 *
 * @param {Date} [date] - Date to form serial or current date by default
 * @returns {string}
 */
var dateBasedSerial = function dateBasedSerial() {
  var date = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : new Date();
  return dateSpread(date).reduce(function (acc, e) {
    return '' + acc + e;
  }, "");
};

/**
 * Array representation of a date: [YYYY, MM, DD, HH, mm]
 *
 * @example
 * dateSpread(new Date(2018, 1, 2, 3, 4)); //=> ['2018', '02', '02', '03', '04']
 *
 * @param {Date} [date] - Instance of Date. If not provided, date will be Date.now()
 * @param {boolean} [padded] - Defines if response will be padded with zeroes
 * @returns {Array}
 */
var dateSpread = function dateSpread() {
  var date = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : new Date();
  var padded = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
  return ["" + date.getFullYear(), leftpad(padded ? 2 : 0, 0, date.getMonth() + 1), leftpad(padded ? 2 : 0, 0, date.getDate()), leftpad(padded ? 2 : 0, 0, date.getHours()), leftpad(padded ? 2 : 0, 0, date.getMinutes())];
};

/**
 * First date of year in YYYY-MM-DD format
 *
 * @example
 * firstDayOfYear(new Date(2018, 1, 2, 3, 4)); //=> '2018-01-01'
 *
 * @param {Date} [date] - Instance of Date. If not provided, date will be Date.now()
 * @returns {string}
 */
var firstDayOfYear = function firstDayOfYear() {
  var date = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : new Date();

  return date.getFullYear() + '-01-01';
};

/**
 * Transform a date range into a readable format
 *
 * @example
 * formatDateRange("2018-03-01", "2018-03-31"); //=> "Mar 2018"
 * formatDateRange("2018-03-01", "2018-03-17"); //=> "1 - 17 Mar 2018"
 * formatDateRange("2018-02-23", "2018-03-17"); //=> "23 Feb - 17 Mar 2018"
 * formatDateRange("2017-12-20", "2018-01-10"); //=> "20 Dic - 10 Ene 2018"
 *
 * @param {string} date_gte - Lowest date of range in YYYY-MM-DD format
 * @param {string} date_lte - Highest date of range in YYYY-MM-DD format
 * @returns {string}
 */
var formatDateRange = function formatDateRange(date_gte, date_lte) {
  var d_gte = new Date(date_gte.replace(/-/g, '\/')); //lowest
  var d_lte = new Date(date_lte.replace(/-/g, '\/')); //highest

  if (d_gte.getMonth() === d_lte.getMonth()) {
    if (d_gte.getDate() === 1 && d_lte.getDate() === getDaysOfMonth(d_gte)) {
      return MONTHS_SHORT.mx[d_gte.getMonth()] + ' ' + d_gte.getFullYear();
    } else {
      return d_gte.getDate() + ' - ' + d_lte.getDate() + ' ' + MONTHS_SHORT.mx[d_gte.getMonth()] + ' ' + d_gte.getFullYear();
    }
  } else {
    return d_gte.getDate() + ' ' + MONTHS_SHORT.mx[d_gte.getMonth()] + ' - ' + d_lte.getDate() + ' ' + MONTHS_SHORT.mx[d_lte.getMonth()] + ' ' + d_lte.getFullYear();
  }
};

/**
 * Formats a date into a YYYY-MM-DD string
 *
 * @example
 * formatDateYYYY_MM_DD(new Date(2018, 1, 2, 3, 4)); //=> '2018-02-02'
 *
 * @param {Date} [date] - Date. If not provided, `Date.now()`
 * @returns {string}
 */
var formatDateYYYY_MM_DD = function formatDateYYYY_MM_DD() {
  var date = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : new Date();

  var _dateSpread = dateSpread(date),
      _dateSpread2 = _slicedToArray(_dateSpread, 3),
      year = _dateSpread2[0],
      month = _dateSpread2[1],
      day = _dateSpread2[2];

  return year + '-' + month + '-' + day;
};

/**
 * Formats a date into a `DD de MMMM YYYY` readable string
 *
 * @example
 * formatDayMonthYear(new Date(2018, 1, 2, 3, 4)); //=> '02 de Febrero 2018'
 *
 * @param {Date} [date] - Date. If not provided, `Date.now()`
 * @param {string} [locale] - Locality string. `mx` by default
 * @returns {string}
 */
var formatDayMonthYear = function formatDayMonthYear() {
  var date = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : new Date();
  var locale = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'mx';

  var _dateSpread3 = dateSpread(date, false),
      _dateSpread4 = _slicedToArray(_dateSpread3, 3),
      year = _dateSpread4[0],
      month = _dateSpread4[1],
      day = _dateSpread4[2];

  return day + ' de ' + MONTHS[locale][month - 1] + ' ' + year;
};

/**
 * Formats a date into a `DD de MMM YY` readable string
 *
 * @example
 * formatDayMonthYearShort(new Date(2018, 1, 2, 3, 4)); //=> '02 Feb 18'
 *
 * @param {Date} [date] - Date. If not provided, `Date.now()`
 * @param {string} [locale] - Locality string. `mx` by default
 * @returns {string}
 */
var formatDayMonthYearShort = function formatDayMonthYearShort() {
  var date = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : new Date();
  var locale = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'mx';

  var _dateSpread5 = dateSpread(date, false),
      _dateSpread6 = _slicedToArray(_dateSpread5, 3),
      year = _dateSpread6[0],
      month = _dateSpread6[1],
      day = _dateSpread6[2];

  return day + ' ' + MONTHS_SHORT[locale][month - 1] + ' ' + year % 100;
};

/**
 * Formats a date into a `MMM YY` readable string
 *
 * @example
 * formatMonthYearShort(new Date(2018, 1, 2, 3, 4)); //=> 'Feb 18'
 *
 * @param {Date} [date] - Date. If not provided, `Date.now()`
 * @param {string} [locale] - Locality string. `mx` by default
 * @returns {string}
 */
var formatMonthYearShort = function formatMonthYearShort() {
  var date = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : new Date();
  var locale = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'mx';

  var _dateSpread7 = dateSpread(date, false),
      _dateSpread8 = _slicedToArray(_dateSpread7, 2),
      year = _dateSpread8[0],
      month = _dateSpread8[1];

  return MONTHS_SHORT[locale][month - 1] + ' ' + year % 100;
};

/**
 * Give the count of how many days a month has. Supports leap-years
 *
 * @example
 * getDaysOfMonth(new Date(2018, 1, 2, 3, 4)); //=> 28
 *
 * @param {Date} [date] - Date. If not provided, `Date.now()`
 * @returns {number}
 */
var getDaysOfMonth = function getDaysOfMonth() {
  var date = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : new Date();

  var year = date.getFullYear();
  var month = date.getMonth();
  return 32 - new Date(year, month, 32).getDate();
};

/**
 *  Get Coordinated Universal Time
 *  - Why I might want to use this? `new Date()` adds a timezone offset. So `new Date('2018','01','01'); //=> 2018-02-01T06:00:00.000Z`.
 *  - Notice the 6 hours added in the result? That is the timezone. To avoid this, we can use UTC format. Hence, this function.
 *
 * @param {Date} date - Instance of `Date`
 * @returns {Date}
 */
var getUTCDate = function getUTCDate(date) {
  var _dateSpread9 = dateSpread(date),
      _dateSpread10 = _toArray(_dateSpread9),
      year = _dateSpread10[0],
      month = _dateSpread10[1],
      rest = _dateSpread10.slice(2);

  return new Date(Date.UTC.apply(Date, [year, month - 1].concat(_toConsumableArray(rest))));
};

/**
 * Validates that an input is instance of Date
 *
 * @example
 * isDate(new Date()); //=> true
 *
 * @param {*} date - Element to check if is instance of Date
 * @returns {boolean}
 */
var isDate = function isDate(date) {
  return date instanceof Date;
};

/**
 * Generates the date_range of the month given in date
 *
 * @example
 * monthRange(new Date('2018', '06')); //=> {from: '2018-05-01', to: '2018-05-31'}
 *
 * @param {Date} [date] - Instance of `Date`. Date.now() if not provided
 * @returns {Object}
 */
var monthRange = function monthRange() {
  var date = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : new Date();

  var _dateSpread11 = dateSpread(date, false),
      _dateSpread12 = _slicedToArray(_dateSpread11, 2),
      year = _dateSpread12[0],
      month = _dateSpread12[1];

  var _dateSpread13 = dateSpread(new Date(year, month, 0)),
      _dateSpread14 = _slicedToArray(_dateSpread13, 3),
      _year = _dateSpread14[0],
      padded_month = _dateSpread14[1],
      lday = _dateSpread14[2];

  return {
    from: year + '-' + padded_month + '-01',
    to: year + '-' + padded_month + '-' + lday
  };
};

/**
 * Generates the `Date` of yesterday ¯\\\_(ツ)\_/¯
 *
 * @returns {Date}
 */
var yesterday = function yesterday() {
  var a_day_before = new Date();
  a_day_before.setDate(a_day_before.getDate() - 1);

  return a_day_before;
};

/**
 * Clock class to transform a `float` represented time into different formats
 *
 * @class
 *
 * @example
 * let clock = new Clock(37.32454);
 * //=> clock = Clock { time: 37.32454, hours: 37, minutes: 19, seconds: 28 }
 * clock.hh_mm_ss(); //=> 37:19:28
 * clock.verbose_hrs_min(); //=> 37h 19m
 * clock.verbose_hrs_min_secs(); //=> 37h 19m 28s
 *
 * @param {number} - Float number representing a time
 * @return {Clock}
 * */

var Clock = function () {
  function Clock() {
    var time = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;

    _classCallCheck(this, Clock);

    this.time = time;
    this.hours = Math.floor(this.time);
    this.minutes = Math.floor(this.time % 1 * 60);
    this.seconds = Math.round(this.time * 3600 % 60);
  }

  _createClass(Clock, [{
    key: 'hh_mm_ss',
    value: function hh_mm_ss() {
      var hh = leftpad(2, "0", this.hours);
      var mm = leftpad(2, "0", this.minutes);
      var ss = leftpad(2, "0", this.seconds);

      return hh + ':' + mm + ':' + ss;
    }
  }, {
    key: 'verbose_hrs_min',
    value: function verbose_hrs_min() {
      var hh = this.hours;
      var mm = this.minutes;
      var ss = this.seconds;

      return hh + 'h ' + mm + 'm';
    }
  }, {
    key: 'verbose_hrs_min_secs',
    value: function verbose_hrs_min_secs() {
      var hh = this.hours;
      var mm = this.minutes;
      var ss = this.seconds;

      return hh + 'h ' + mm + 'm ' + ss + 's';
    }
  }]);

  return Clock;
}();

module.exports = {
  MONTHS: MONTHS,
  MONTHS_SHORT: MONTHS_SHORT,
  WEEKDAYS_LONG: WEEKDAYS_LONG,
  WEEKDAYS_SHORT: WEEKDAYS_SHORT,

  dateBasedSerial: dateBasedSerial,
  dateSpread: dateSpread,
  firstDayOfYear: firstDayOfYear,
  formatDateRange: formatDateRange,
  formatDateYYYY_MM_DD: formatDateYYYY_MM_DD,
  formatDayMonthYear: formatDayMonthYear,
  formatDayMonthYearShort: formatDayMonthYearShort,
  formatMonthYearShort: formatMonthYearShort,
  getDaysOfMonth: getDaysOfMonth,
  getUTCDate: getUTCDate,
  isDate: isDate,
  monthRange: monthRange,
  yesterday: yesterday,

  Clock: Clock
};